package org.rayflood.mikuvat.io;

import static org.rayflood.mikuvat.io.VSQDecoder.getByteArrayString;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.sound.midi.MidiMessage;
import javax.sound.midi.SysexMessage;

public class MidiSysex extends MidiData{
	private static final long serialVersionUID = -697221293558971797L;
	private int status;
	private byte[] data;

	public MidiSysex(MidiMessage mm){
		setMidiEvent(mm);
	}

	public MidiSysex(int status, byte[] data){
		setStatus(status);
		setData(data);
	}

	public SysexMessage getMidiEvent() throws Exception{
		SysexMessage sysex = new SysexMessage();
		sysex.setMessage(getStatus(), getData(), getData().length);
		return sysex;
	}

	public void setMidiEvent(MidiMessage mm){
		SysexMessage sysex = (SysexMessage)mm;
		setStatus(sysex.getStatus());
		setData(sysex.getData());
	}

	public String toString(){
		List<String> sl = new ArrayList<String>();
		sl.add("Status=" + getStatus());
		sl.add("Data=" + getByteArrayString(getData()));

		StringBuffer sb = new StringBuffer();
		sb.append("MidiSysex={");
		if(!sl.isEmpty()){
			sb.append(sl.get(0));
			for(int i = 1; i < sl.size(); i++){
				sb.append("," + sl.get(i));
			}
		}
		sb.append("}");
		return sb.toString();
	}

	public boolean isReadied(){
		return true;
	}

	public int compareTo(ImmutableObject obj){
		MidiSysex midiSysex = (MidiSysex)obj;
		return compare(getStatus(), midiSysex.getStatus());
	}

	public MidiSysex over(Object obj, boolean fill, boolean has){
		MidiSysex midiSysex = ((MidiSysex)obj).clone();
		setStatus(midiSysex.getStatus());
		setData(midiSysex.getData());
		return this;
	}

	public MidiSysex setDefault(){
		return this;
	}

	public static MidiSysex getDefault(){
		MidiSysex midiSysex = new MidiSysex();
		midiSysex.setDefault();
		return midiSysex;
	}

	public MidiSysex delete(){
		setStatus(0);
		setData(new byte[]{});
		return this;
	}

	public boolean isDeleted(){
		return false;
	}

	public MidiSysex(){
	}

	public int getStatus(){
		return status;
	}

	public void setStatus(int status){
		this.status = status;
	}

	public byte[] getData(){
		return data;
	}

	public void setData(byte[] data){
		this.data = data;
	}

	public MidiSysex clone(){
		return (MidiSysex)super.clone();
	}

	public static MidiSysex load(String file) throws Exception{
		return (MidiSysex)ImmutableObject.load(file);
	}

	public static MidiSysex load(File file) throws Exception{
		return (MidiSysex)ImmutableObject.load(file);
	}

	public void store(String file) throws Exception{
		super.store(file);
	}

	public void store(File file) throws Exception{
		super.store(file);
	}

	public static MidiSysex unzip(String file) throws Exception{
		return (MidiSysex)ImmutableObject.unzip(file);
	}

	public static MidiSysex unzip(File file) throws Exception{
		return (MidiSysex)ImmutableObject.unzip(file);
	}

	public void zip(String file) throws Exception{
		super.zip(file);
	}

	public void zip(File file) throws Exception{
		super.zip(file);
	}
}
